from .models import Original
from django.db.models.signals import pre_save
import django_rq
from django.db import models
import io
import requests


@django_rq.job
def create_renditions(renditions, parent, original, output):
    try:
        Original.objects.get(pk=original.pk)
    except Original.DoesNotExist as e:
        raise e
    original_file = io.BytesIO(requests.get(original.href).content)
    for rendition_name, rendition_obj in renditions.items():
        # returns an unsaved rendition object
        rendition_obj = rendition_obj.create_rendition(original_file=original_file, output=output)
        rendition_obj.original = original
        rendition_obj.name = rendition_name
        rendition_obj.save(force_insert=True)
    original.ready = True
    original.save()


class FileUrlWithRenditions(models.OneToOneField):

    def __init__(self, renditions=None, source=None, output=None, use_job_runner=False, to=Original, null=True, **kwargs):
        self.renditions = renditions
        self.source = source
        self.use_job_runner = use_job_runner
        self.output = output
        kwargs['on_delete'] = models.SET_NULL
        kwargs['blank'] = True
        kwargs['null'] = True
        super(FileUrlWithRenditions, self).__init__(to=to, **kwargs)

    def handle_source_pre_save(self, sender, instance, **kwargs):
        source_file = getattr(instance, self.source)
        original_obj = getattr(instance, self.name)
        if not original_obj or original_obj.href != source_file:
            if original_obj:
                original_obj.delete()
                # FIXME: WHY ? maybe original_obj is not what it was when we used post_save
                setattr(instance, self.name, None)
            if not source_file:
                return
            original_obj = Original.objects.create(href=source_file)
            setattr(instance, self.name, original_obj)
            # create renditions
            props = dict(renditions=self.renditions, output=self.output, parent=instance, original=original_obj)
            if self.use_job_runner:
                create_renditions.delay(**props)
            else:
                create_renditions(**props)

    def contribute_to_class(self, cls, name):
        super(FileUrlWithRenditions, self).contribute_to_class(cls, name)
        self.remote_field.related_name = '{}_{}'.format(cls.__name__, name)
        pre_save.connect(self.handle_source_pre_save, sender=cls)



from django.conf import settings
import os
import io
from url_renditions.models import Rendition
from url_renditions.render import Render
import uuid
from pydub import AudioSegment
import boto3


s3 = boto3.resource('s3')


def generate_mp3(original_file):
    audio = AudioSegment.from_file(original_file)
    out_audio = io.BytesIO()
    audio.export(out_audio)
    out_audio.seek(0)
    return out_audio


def upload_mp3(audio_bin, output):
    uploaded = s3.Bucket(settings.AWS_STORAGE_BUCKET_NAME).put_object(
        ACL='public-read',
        Key=os.path.join(output, uuid.uuid4().hex + '.mp3'),
        Body=audio_bin,
        ContentDisposition='inline',
        ContentType='audio/mp3',
    )
    return 'https://s3.{}.amazonaws.com/{}/{}'.format(
        settings.S3DIRECT_REGION,
        settings.AWS_STORAGE_BUCKET_NAME,
        uploaded.key,
    )


class RenderMp3(Render):

    def create_rendition(self, original_file, output):
        mp3_bin = generate_mp3(original_file)
        href = upload_mp3(mp3_bin, output)
        return Rendition(href=href)



from django.db import models


class Original(models.Model):
    href = models.URLField()
    width = models.IntegerField(null=True, blank=True)
    height = models.IntegerField(null=True, blank=True)
    ready = models.BooleanField(default=False)

    def __str__(self):
        return self.href


class Rendition(models.Model):
    original = models.ForeignKey(Original)
    name = models.CharField(max_length=255)
    href = models.URLField()
    width = models.IntegerField(null=True, blank=True)
    height = models.IntegerField(null=True, blank=True)

    def __str__(self):
        return self.name



# from .render import Render
# from .render_mp3 import RenderMp3
# from .resize_image import ResizeImage
# from .fields import FileUrlWithRenditions

# __all__ = [
#     'Render',
#     'RenderMp3',
#     'ResizeImage',
#     'FileUrlWithRenditions',
# ]



import io
import boto3
from django.conf import settings
import os
import PIL
import PIL.ImageOps
from url_renditions.models import Rendition
from url_renditions.render import Render
import uuid


s3 = boto3.resource('s3')


def resize_and_upload(input, output, width, height):
    """ reduce image resolution to <constant> precent of what is uploaded """
    image = PIL.Image.open(input)
    if image.mode == "CMYK":
        image = image.convert("RGB")
    if height:
        # crop
        new_image = PIL.ImageOps.fit(
            image,
            (height, width),
            PIL.Image.ANTIALIAS
        )
    else:
        wpercent = (width / float(image.size[0]))
        height = int((float(image.size[1]) * float(wpercent)))
        new_image = image.resize((width, height), PIL.Image.ANTIALIAS)

    out_img = io.BytesIO()
    new_image.save(out_img, 'PNG')
    out_img.seek(0)
    uploaded = s3.Bucket(settings.AWS_STORAGE_BUCKET_NAME).put_object(
        ACL='public-read',
        Key=os.path.join(output, uuid.uuid4().hex + '.png'),
        Body=out_img,
        ContentDisposition='inline',
        ContentType='image/png',
    )
    return {
        'href': 'https://s3.{}.amazonaws.com/{}/{}'.format(
            settings.S3DIRECT_REGION,
            settings.AWS_STORAGE_BUCKET_NAME,
            uploaded.key,
        ),
        'width': width,
        'height': height,
    }


class ResizeImage(Render):

    def __init__(self, size):
        width, height = [d and int(d) or None for d in size.split('x')]
        self.width = width
        self.height = height

    def create_rendition(self, original_file, output):
        rendition_s3 = resize_and_upload(
            input=original_file,
            output=output,
            width=self.width,
            height=self.height,
        )
        return Rendition(**rendition_s3)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.8 on 2018-05-04 09:08
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Original',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('href', models.URLField()),
                ('width', models.IntegerField(blank=True, null=True)),
                ('height', models.IntegerField(blank=True, null=True)),
                ('ready', models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name='Rendition',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('href', models.URLField()),
                ('width', models.IntegerField(blank=True, null=True)),
                ('height', models.IntegerField(blank=True, null=True)),
                ('original', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='url_renditions.Original')),
            ],
        ),
    ]



from django.core.management.base import BaseCommand
from django.apps import apps
from url_renditions.fields import FileUrlWithRenditions


class Command(BaseCommand):
    help = 'Reset renditions'

    def add_arguments(self, parser):
        parser.add_argument('models', nargs='+', type=str)

    def handle(self, *args, **options):
        for model in options['models']:
            self.stdout.write('Model {}'.format(model))
            Model = apps.get_model(model)
            fields_to_reset = [f for f in Model._meta.get_fields() if type(f) is FileUrlWithRenditions]
            for o in Model.objects.all():
                to_be_saved = False
                for field in fields_to_reset:
                    if getattr(o, field.source):
                        to_be_saved = True
                        self.stdout.write('reset {}'.format(o))
                        if getattr(o, field.name):
                            getattr(o, field.name).delete()
                            setattr(o, field.name, None)
                if to_be_saved:
                    o.save()



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-url-renditions',
    version='0.2',
    packages=find_packages(),
    include_package_data=True,
    license='BSD License',
    description='Create custom renditions from django url fields',
    long_description=README,
    url='https://vied12.github.io',
    author='Edouard Richard',
    author_email='edou4rd@gmail.com',
    install_requires=['Pillow==5.1.0',],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
)



import graphene
from graphene_django import DjangoObjectType
from . import models
from django.db.models.fields.reverse_related import OneToOneRel


class Rendition(DjangoObjectType):
    class Meta:
        model = models.Rendition
        filter_fields = ['id', 'name']
        interfaces = (graphene.relay.Node, )


def call_that_returns_fields_dict():
    result = {}
    for name in ['small', 'medium']:
        result[name] = graphene.Field(Rendition)
    return result


def get_fields_dict():
    rendition_names = []
    for field in models.Original._meta.get_fields():
        if type(field) is OneToOneRel:
            rendition_names += field.field.renditions.keys()
    result = {}
    for name in set(rendition_names):
        result[name] = graphene.Field(Rendition)
    return result


Renditions = type('Renditions', (graphene.ObjectType,), get_fields_dict())


class Original(DjangoObjectType):

    renditions = graphene.Field(Renditions)

    class Meta:
        model = models.Original
        filter_fields = ['id']
        interfaces = (graphene.relay.Node, )

    def resolve_renditions(self, info, **kwargs):
        renditions = {}
        for r in self.rendition_set.all():
            renditions[r.name] = r
        return Renditions(**renditions)


class Query(object):
    original = graphene.relay.Node.Field(Original)
    rendition = graphene.relay.Node.Field(Rendition)
